#!/bin/bash
# source run_all.sh
#
echo Running examples in directory other/trimesh/

echo Running nearest.py
python3 nearest.py

echo Running ray.py
python3 ray.py

echo Running section.py
python3 section.py

echo Running shortest.py
python3 shortest.py
